local viewer
local frammis = {}

function HideBlizzard:ShowText(text, title, width, border, divide)
	local border = border or 8
	local divide = divide or 4

	if not frammis[border] then frammis[border] = {} end
	viewer = frammis[border][divide]
	local suffix = string.format('_%d_%d', border, divide)

	if not viewer then
		viewer = CreateFrame('Frame', 'HideBlizzardViewer', UIParent)
		viewer:SetHeight(25)
		viewer:SetBackdrop({
			bgFile = 'Interface/Tooltips/UI-Tooltip-Background',
			edgeFile = 'Interface/Tooltips/UI-Tooltip-Border',
			tile = true,
			tileSize = 16,
			edgeSize = 14,
			insets = {left = 4, right = 4, top = 4, bottom = 4}
		})
		viewer:SetBackdropColor(1, 0.5, 0.1, 1)
		viewer:SetBackdropBorderColor(0.4, 0.4, 0.4, 1)
		viewer:SetFrameStrata('HIGH')
		viewer:SetPoint('CENTER', UIParent, 0, 250)
		viewer:EnableMouse(true)
		viewer:SetMovable(true)
		viewer:SetClampedToScreen(true)
		viewer:RegisterForDrag('LeftButton')
		viewer:SetScript('OnDragStart', function(self) self:StartMoving() end)
		viewer:SetScript('OnDragStop', function(self) self:StopMovingOrSizing() end)
		table.insert(UISpecialFrames, viewer:GetName())
		viewer:Hide()

		viewer.minimize = CreateFrame('Button', 'HideBlizzardViewer_MinimizeButton', viewer)
		viewer.minimize:SetWidth(30); viewer.minimize:SetHeight(30)
		viewer.minimize:SetFrameStrata('HIGH')
		viewer.minimize:EnableMouse(true)

		viewer.minimizetex = viewer:CreateTexture(nil, 'ARTWORK')
		viewer.minimize:Show()
		viewer.minimizetex:SetTexture('Interface\\Minimap\\UI-Minimap-MinimizeButtonDown-Up')
		viewer.minimize:SetPushedTexture('Interface\\Minimap\\UI-Minimap-MinimizeButtonDown-Down')
		viewer.minimize:SetHighlightTexture('Interface\\Minimap\\UI-Minimap-MinimizeButtonDown-Highlight')
		viewer.minimize:SetPoint('TOPRIGHT', -22, 3)
		viewer.minimizetex:SetAllPoints(viewer.minimize)
		viewer.minimize:SetScript('OnClick', function() 
			if viewer.main:IsVisible() then 
				viewer.main:Hide()
				viewer.minimizetex:SetTexture('Interface\\Minimap\\UI-Minimap-MinimizeButtonUp-Up')
				viewer.minimize:SetPushedTexture('Interface\\Minimap\\UI-Minimap-MinimizeButtonUp-Up')
				viewer.minimize:SetHighlightTexture('Interface\\Minimap\\UI-Minimap-MinimizeButtonUp-Highlight')
			else
				viewer.main:Show()
				viewer.minimizetex:SetTexture('Interface\\Minimap\\UI-Minimap-MinimizeButtonDown-Up')
				viewer.minimize:SetPushedTexture('Interface\\Minimap\\UI-Minimap-MinimizeButtonDown-Down')
				viewer.minimize:SetHighlightTexture('Interface\\Minimap\\UI-Minimap-MinimizeButtonDown-Highlight')
			end
		end)

		viewer.close = CreateFrame('Button', 'HideBlizzardViewer_CloseButton', viewer)
		viewer.close:SetWidth(30); viewer.close:SetHeight(30)
		viewer.close:SetFrameStrata('HIGH')
		viewer.close:EnableMouse(true)

		viewer.closetex = viewer:CreateTexture(nil, 'ARTWORK')
		viewer.close:Show()
		viewer.closetex:SetTexture('Interface\\BUTTONS\\UI-Panel-MinimizeButton-Up')
		viewer.close:SetPushedTexture('Interface\\BUTTONS\\UI-Panel-MinimizeButton-Down')
		viewer.close:SetHighlightTexture('Interface\\BUTTONS\\UI-Panel-MinimizeButton-Highlight')
		viewer.closetex:SetAllPoints(viewer.close)
		viewer.close:SetPoint('TOPRIGHT', viewer,-2, 3)
		viewer.close:SetScript('OnClick', function() viewer:Hide() end)

		viewer.title = viewer:CreateFontString()
		viewer.title:SetFont('Fonts\\FRIZQT__.TTF', 16)
		viewer.title:SetJustifyV('CENTER'); viewer.title:SetJustifyH('CENTER')
		viewer.title:SetPoint('CENTER', viewer, 0, 0)

		viewer.main = CreateFrame('Frame', 'HideBlizzardViewer_MainFrame', viewer)
		viewer.main:SetBackdrop({
			bgFile = 'Interface/Tooltips/UI-Tooltip-Background',
			edgeFile = 'Interface/Tooltips/UI-Tooltip-Border',
			edgeSize = 16,
			tile = true,
			tileSize = 16,
			insets = {left = 4, right = 4, top = 4, bottom = 4}
		})
		viewer.main:SetBackdropColor(0, 0, 0, 1)
		viewer.main:SetBackdropBorderColor(0.4, 0.4, 0.4, 1)
		viewer.main:SetFrameStrata('HIGH')
		viewer.main:SetPoint('TOP', viewer, 0, -22)

		viewer.main.scrollframe = CreateFrame('ScrollFrame', 'HideBlizzardViewer_ScrollFrame'..suffix, viewer.main, 'UIPanelScrollFrameTemplate')
		viewer.main.scrollframe:SetPoint("LEFT", viewer.main, "LEFT", border, 0)
		viewer.main.scrollframe:SetPoint("TOP", viewer.main, "TOP", 0, -divide)

		viewer.main.scrollbar = _G['HideBlizzardViewer_ScrollFrame'..suffix..'ScrollBar']
		viewer.main.scrollbar:SetBackdrop({
			edgeFile = 'Interface/Tooltips/UI-Tooltip-Border',
			edgeSize = 12,
			tileSize = 16,
			insets = {left = 0, right = 0, top = 5, bottom = 5}
		 })
 
		viewer.frame = CreateFrame('Frame', 'HideBlizzardViewer_Frame'..suffix, viewer.main.scrollframe)
		viewer.main.scrollframe:SetScrollChild(viewer.frame)

		viewer.frame.text = viewer.frame:CreateFontString()
		viewer.frame.text:SetFont('Fonts\\FRIZQT__.TTF', 14)
		viewer.frame.text:SetJustifyH('LEFT')
		viewer.frame.text:SetPoint('TOPLEFT', viewer.frame, 5, -3)
	end

	viewer:Show()
	viewer.title:SetText(title or 'HideBlizzard')
	viewer.frame.text:SetText(text or 'No Text')
	viewer.main.scrollframe:SetVerticalScroll(0)

	local w = width or math.min(600, math.max(100, viewer.frame.text:GetStringWidth()))
	viewer.frame.text:SetWidth(w)
	viewer.main:SetWidth(w+border * 2)
	viewer.main.scrollframe:SetWidth(w)
	viewer.frame:SetWidth(w)
	viewer:SetWidth(viewer.main:GetWidth())

	local h = math.max(10, viewer.frame.text:GetHeight())
	local title_h = viewer.main:GetHeight()

	if h > 400 then
		viewer.frame:SetHeight(400)
		viewer.main.scrollframe:SetHeight(400)
		viewer.main:SetHeight(420+title_h+border * 2+divide)
		viewer.main:SetWidth(w+border * 2 + 22)
		viewer.main.scrollbar:Show()
	else
		viewer.frame:SetHeight(h)
		viewer.main.scrollframe:SetHeight(h)
		viewer.main:SetHeight(h+border * 2 + divide+title_h)
		viewer.main.scrollbar:Hide()
	end
	frammis[border][divide] = viewer
end